@@ -31,7 +31,7 @@ rd_insert_na <- function(..., data = NULL, dic = NULL, event_form = NULL, vars,
     }
 
     data <- project$data
-    dic <- project$dic
+    dic <- project$dictionary
 
     if("event_form" %in% names(project)){
       if(!is.null(event_form)){
